;	Star Bars Player

QUALITY		equ	$07		; $03 = high, $07 = good, $0f = average, $1f = low

SoundData	equ	$f500
SequencerData	equ	$f600
PatternData	equ	$f700

PALTempo	equ	$f505
NTSCTempo	equ	$f507

basenote	equ	$80		; c0, c1 base note as read from pattern (2 bytes)
sndpos		equ	$82		; c0, c1 sound data pointer (2 bytes)
pattpos		equ	$84		; pattern position ($00-$3e)
tempo		equ	$85		; current tempo
countdown	equ	$86		; step timing countdown
temp		equ	$87		; temporary
c0pattern	equ	$88		; 16-bit pointer to start of current c0 pattern
c1pattern	equ	$8a		; 16-bit pointer to start of current c1 pattern
seqpos		equ	$8c		; sequencer position ($00-$fe)

c0freq		equ	$8d		; tiatonic tuning
c1freq		equ	$8e
c0bits		equ	$8f
c1bits		equ	$90
maskbit		equ	$91

NOERRORCHECKS	equ	0		; set to 1 to save 16 bytes and a few cycles by skipping the checks for
					; invalid sound/sequencer loops and arpeggio notes going out of range



;	Tiatonic Tuning
;
;	Assumes scanline counter in x and rotates maskbit (used by Star Bars ROM export and test player)


	MAC	TiatonicTuning

.scanlinedivider
	txa			; scanline counter in x
	and	#QUALITY	; fine tuning every 4-32th line (use $03, $07, $0f, $1f)
	bne	.done

.c0tuning
	asl	maskbit		; rotate mask bit
	bcc	.1
	inc	maskbit
.1
	lda	c0bits		; channel 0 freq tuning bits
	ldy	c0freq		; channel 0 freq
	and	maskbit
	beq	.2
	iny
.2
	sty	AUDF0

.c1tuning
	lda	c1bits		; channel 1 freq tuning bits
	ldy	c1freq		; channel 1 freq
	and	maskbit
	beq	.3
	iny
.3
	sty	AUDF1
.done

	ENDM


;	Player Init

PlayerInit	subroutine
	IF PAL == 1
	lda	PALTempo
	ELSE
	lda	NTSCTempo
	ENDIF
	sta	tempo
	lda	#2			; new pattern step after 2 frames
	sta	countdown

	lda	#$80			; init tiatonic tuning rotating mask bit
	sta	maskbit
	bne	SequencerStep		; get first patterns from sequencer


;	Player Update

Player	subroutine
	dec	countdown		; pattern step timing
	beq	.step			; next step if countdown = 0
	lda	tempo
	lsr
	cmp	countdown		; or if countdown = tempo/2
	beq	.halfstep

	ldx	#1			; update sounds
	bne	SoundUpdate

.step
	lda	tempo			; reset step timer to tempo
	sta	countdown
.halfstep
	ldy	pattpos
	lda	(c0pattern),y		; get c0 note from pattern
	beq	.nonotec0
	bmi	SequencerStep		; pattern break
	sta	basenote
	iny
	lda	(c0pattern),y
	dey

	asl
	bcs	.nonotec0		; bit 7 set = tied note, change basenote only
	sta	sndpos

.nonotec0
	lda	(c1pattern),y		; get c1 note from pattern
	beq	.nonotec1
	bmi	SequencerStep		; pattern break
	sta	basenote+1
	iny
	lda	(c1pattern),y
	dey

	asl
	bcs	.nonotec1		; bit 7 set = tied note, change basenote only
	sta	sndpos+1

.nonotec1
	iny
	iny
	cpy	#$40
	bne	.nowrap
	;

SequencerStep
	ldy	seqpos

	lda	SequencerData,y		; get c0 pattern from sequencer
	bpl	.noloop

	asl				; loop position
	sta	seqpos
	IF NOERRORCHECKS == 1
	bcs	SequencerStep		; carry always set from asl
	ELSE
	tay
	lda	SequencerData,y		; skip loop command after loop to avoid possible hangup
	ENDIF

.noloop
	ldx	#$00			; x 64
	stx	temp
	lsr
	ror	temp
	lsr
	ror	temp
	adc	#PatternData>>8		; + $f700 (carry is clear)
	ldx	temp
	stx	c0pattern
	sta	c0pattern+1

	lda	SequencerData+1,y	; get c1 pattern from sequencer

	ldx	#$00			; x 64
	stx	temp
	lsr
	ror	temp
	lsr
	ror	temp
	adc	#PatternData>>8		; + $f700 (carry is clear)
	ldx	temp
	stx	c1pattern
	sta	c1pattern+1

	iny
	iny
	sty	seqpos

	ldy	#0
.nowrap
	sty	pattpos

	ldx	#1			; update sounds
	;

SoundUpdate
	lda	sndpos,x		; sound pointer
	and	#$fe
	sta	temp
	tay
	lda	SoundData,y		; arpeggio
	bmi	.noiseorloop
.afterloop
	cmp	#$40
	bcs	.absolutenote
	;
.relativearp
	adc	basenote,x		; relative arpeggio base note + arpeggio (carry is clear)

	IF NOERRORCHECKS == 0
	cmp	#74
	bcc	.pulse
	lda	#0			; silence if arpeggio goes above c-7
	ENDIF
.pulse
	tay
	lda	freqtab,y
	sta	c0freq,x
	lda	freqtab+$50,y
	sta	c0bits,x
	lda	freqtab+$a0,y
.setwaveform
	sta	AUDC0,x

.getvolume
	ldy	temp			; sound pointer
	lda	SoundData+1,y		; volume
	sta	AUDV0,x
	bmi	.slower			; if bit 7 set in envelope volume then advance in sound data at half speed

	inc	sndpos,x		; next step in sound
.slower
	inc	sndpos,x

	dex				; update the other channel
	bpl	SoundUpdate

	rts

.absolutenote
	and	#$3f
	bpl	.pulse

.noiseorloop
	cmp	#$f0
	bcs	.soundloop
.noise
	sta	c0freq,x		; noise pitch $00-$1f
	lda	#$00			; reset tuning bits
	sta	c0bits,x
	lda	#$08			; noise waveform
	bne	.setwaveform		; always branch

.soundloop
	and	#$0f
	asl				; carry cleared
	sta	temp
	lda	sndpos,x
	and	#$e0
	ora	temp
	sta	sndpos,x

	IF NOERRORCHECKS == 1
	bcc	SoundUpdate
	ELSE
	sta	temp
	tay
	lda	SoundData,y		; skip loop command after loop to avoid possible hangup
	bmi	.noise
	bcc	.afterloop		; always branch
	ENDIF

